#include "categorytree.h"
#include "categoryitem.h"
#include "articleitem.h"
#include "pager.h"

#include <qlistview.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qvaluelist.h>
#include <qheader.h>
#include <stdio.h>
#include <iostream>


CCategoryTree::CCategoryTree( QWidget *parent, const char *name)
  : QListView (parent, name )
{

  m_nVersion = 0;

  addColumn( QString(""));

  setRootIsDecorated(true);
  header()->hide();

}

void CCategoryTree::slotLoadCategories()
{
  loadCategories("category.dat");
}

bool CCategoryTree::saveCategories(const QString &strFilename)
{
  QFile f(strPagerDir + strFilename);
  QString strBuffer;
  CCategoryItem *item = (CCategoryItem *)firstChild();

  if(f.open(IO_Truncate | IO_WriteOnly)) {
    QTextStream t(&f);
    t << "VERSION=" << m_nVersion << endl;
    t << "NUMENTRIES=" << m_nEntries << endl;

    // Special case for announcements, which are never displayed in category tree because you cannot turn them off

    t << "20001,0,20001,\"announcements\",1" << endl;
    while(item) {
      item->writeSelf(t);
      item->saveChildren(t);

      item = (CCategoryItem *)item->nextSibling();
    }
    f.close();
    return true;
  }
  return false;
}
    


bool CCategoryTree::loadCategories(const QString &strFilename)
{
  QFile f(strPagerDir + strFilename);
  QFile old(strPagerDir + "category.old");
  QString strBuffer;
  int  nEntryCount = 0;
  int n;
  
  m_nVersion = 0;
  m_nEntries = 0;

  if(f.open(IO_ReadOnly)) {
    this->clear();
    QTextStream t(&f);
    QString s;
    while(!t.eof() ) {
      s = t.readLine();

      s.stripWhiteSpace();
      if(s.isEmpty())
	continue;

      if(s.find("VERSION=", 0) == 0) {
	sscanf(s, "VERSION=%d", &n);
	if(n > 0) 
	  m_nVersion = n;
      }
      
      if(s.find("NUMENTRIES=", 0) == 0) {
	sscanf(s, "NUMENTRIES=%d", &n);
	if(n > 0)
	  m_nEntries = n;
      }

      // Feed input to general parse function
      if(ParseLineFromFile(s))
	nEntryCount++;

    }
    f.close();

    if(nEntryCount != m_nEntries || m_nVersion == 0) {
      QMessageBox::critical(this, "Uh-Oh", "Category file is corrupted");
      qWarning("VERSION = %d, NUMENTRIES = %d, Counted %d entries", m_nVersion, m_nEntries, nEntryCount);
      exit(1);
    } 

    //
    // If category.old exists, we have to strip information about what categories are selected from it and set the selected flag 
    // properly on the corresponding current categories
    //
    if(old.exists() && old.open(IO_ReadOnly)) {
      QTextStream t(&old);
      QString s;
      while(!t.eof()) {
	int nTopLevelID, nParentID, nNodeID, n;
	s = t.readLine();
	s.stripWhiteSpace();
	if(s.isEmpty())
	  continue;
	if(sscanf(s, "%d,%d,%d,", &nTopLevelID, &nParentID, &nNodeID) != 3)
	  continue;
	n = s.findRev("\",");
	if(n == -1)
	  continue;
	n += 2;
	if(s.mid(n).toInt() == 1) {
	  // select this item
	  CCategoryItem *item = GetItemForID(nNodeID, nTopLevelID);
	  if(item) {
	    item->setChecked(true);
	  }
	}
      }
      old.remove();
      saveCategories("category.dat");
    }

    emit categoriesLoaded();
    return true;
  }

  QMessageBox::critical(this, "Oh-No", "Cannot open category file");
  exit(1);
}
  
bool CCategoryTree::ParseLineFromFile(const QString strLine)
{
  int n, nLen;
  int nParentID, nNodeID, nTopLevelID;
  QString strDescription;
  bool bSelected = false;

  //
  // Grab the Category IDs
  //
  if(sscanf(strLine, "%d,%d,%d,", &nTopLevelID, &nParentID, &nNodeID) != 3)
    return false;

  //
  // Locate the description and copy it
  //
  if( (n = strLine.find('"', 0)) == -1)
    return false;

  nLen = strLine.length();

  strDescription = strLine.right(nLen - n - 1);

  //
  // remove everything after trailing double-quote
  //
  n = strDescription.findRev('"');
  if(n != -1)
    strDescription.truncate(n);

  // If selected flag is present process it
  n = strLine.findRev("\",");
  if(n != -1) {
    n += 2;
    if(strLine.mid(n).toInt() == 1)
      bSelected = true;
  }
  //
  // Special case for "announcements"
  // don't add to category tree, but account for entry
  //
  if(nTopLevelID == 20001 && nParentID == 0)
    return true;


  return AddToCategories(strDescription, nTopLevelID, nNodeID, nParentID, bSelected);

}

bool CCategoryTree::AddToCategories(QString strDescription, int nTopLevelID, int nNodeID, int nParentID
				    , bool bSelected)
{
  
  CCategoryItem *item, *pParent = NULL;

  if(nParentID == 0) {
    item = new CCategoryItem(this, strDescription, nTopLevelID, nNodeID, nParentID, true);
    item->setChecked(bSelected);
  }
  else {
    pParent = GetItemForID(nParentID, nTopLevelID);
    if(pParent) {
      item =  new CCategoryItem(pParent, strDescription, nTopLevelID, nNodeID, nParentID, true);
      item->setChecked(bSelected);
    }
    else {
      qWarning("Could not find parent (%d) for item (%d).  TopLevel is %d", nParentID, nNodeID, nTopLevelID);
      return false;
    }
  }
  return true; 
}


CCategoryItem *CCategoryTree::GetItemForID(int nNodeID, int nTopLevel)
{
  CCategoryItem *item = (CCategoryItem *)firstChild();
  
  while(item) {
    if(item->getNodeID() == nTopLevel) {
      if(item->getNodeID() == nNodeID)
	return (item);
      return (item->getItemForID(nNodeID));
    }

    item = (CCategoryItem *)item->nextSibling();
  }

  return 0;
}

//
// Places Top Level categories that have content selected into the content tree 
//
void CCategoryTree::setActiveTopLevel(QListView *pContent)
{

  pContent->clear();

  // Announcements are always added
  new CCategoryItem(pContent, "announcements", 20001, 20001, 0);

  CCategoryItem *item = (CCategoryItem *)firstChild();

  while(item) {
    if(item->isSelected() || item->anyChildrenSelected() )
      (void) new CCategoryItem(pContent, item->text(0), item->getTopLevel(), item->getNodeID(), item->getParentID());

    item = (CCategoryItem *)item->nextSibling();
  }
}

QString CCategoryTree::getContentQuery(int nItems)
{
  QString strTmp, strQuery;
  CCategoryItem *topItem;

  strQuery.sprintf("version=%d&nentries=%d&Content=", m_nVersion, nItems);
  // Announcements are always polled for
  strQuery += "T20001";

  topItem = (CCategoryItem *)firstChild();
  while(topItem) {
    if(topItem->isSelected() || topItem->anyChildrenSelected()) {
	strTmp.sprintf(",T%d", topItem->getNodeID());
	strQuery += strTmp;
	strQuery += topItem->getQueryString();
    }
    topItem = (CCategoryItem *)topItem->nextSibling();
  }

  return strQuery;
}

    









